@@ -19,6 +19,7 @@ from gpt_index.data_structs.struct_type import IndexStructType
 from gpt_index.indices.prompt_helper import PromptHelper
 from gpt_index.indices.query.query_runner import QueryRunner
 from gpt_index.indices.query.schema import QueryConfig, QueryMode
+from gpt_index.indices.response.schema import Response
 from gpt_index.indices.utils import truncate_text
 from gpt_index.langchain_helpers.chain_wrapper import LLMPredictor
 from gpt_index.langchain_helpers.text_splitter import TokenTextSplitter
@@ -150,6 +151,25 @@ class BaseGPTIndex(Generic[IS]):
         """
         self._index_struct.text = text
 
+    def set_doc_id(self, doc_id: str) -> None:
+        """Set doc_id for index struct.
+
+        This is used to uniquely identify the index struct in the docstore.
+        If you wish to delete the index struct, you can use this doc_id.
+
+        """
+        self._index_struct.doc_id = doc_id
+
+    def get_doc_id(self) -> str:
+        """Get doc_id for index struct.
+
+        If doc_id not set, raise an error.
+
+        """
+        if self._index_struct.doc_id is None:
+            raise ValueError("Index must have doc_id property set.")
+        return self._index_struct.doc_id
+
     def _get_nodes_from_document(
         self,
         document: BaseDocument,
@@ -191,7 +211,12 @@ class BaseGPTIndex(Generic[IS]):
 
     @llm_token_counter("insert")
     def insert(self, document: DOCUMENTS_INPUT, **insert_kwargs: Any) -> None:
-        """Insert a document."""
+        """Insert a document.
+
+        Args:
+            document (Union[BaseDocument, BaseGPTIndex]): document to insert
+
+        """
         processed_doc = self._process_documents([document], self._docstore)[0]
         self._validate_documents([processed_doc])
         self._insert(processed_doc, **insert_kwargs)
@@ -201,12 +226,39 @@ class BaseGPTIndex(Generic[IS]):
         """Delete a document."""
 
     def delete(self, doc_id: str, **delete_kwargs: Any) -> None:
-        """Delete a document."""
+        """Delete a document from the index.
+
+        All nodes in the index related to the index will be deleted.
+
+        Args:
+            doc_id (str): document id
+            full_delete (bool): whether to delete the document from the docstore.
+                By default this is True.
+            verbose (bool): whether to print verbose output. By default this is False.
+
+        """
+        verbose = delete_kwargs.pop("verbose", False)
         full_delete = delete_kwargs.pop("full_delete", True)
+        if verbose:
+            print(f"> Deleting document: {doc_id}")
         if full_delete:
             self._docstore.delete_document(doc_id)
         self._delete(doc_id, **delete_kwargs)
 
+    def update(self, document: DOCUMENTS_INPUT, **update_kwargs: Any) -> None:
+        """Update a document.
+
+        This is equivalent to deleting the document and then inserting it again.
+
+        Args:
+            document (Union[BaseDocument, BaseGPTIndex]): document to update
+            insert_kwargs (Dict): kwargs to pass to insert
+            delete_kwargs (Dict): kwargs to pass to delete
+
+        """
+        self.delete(document.get_doc_id(), **update_kwargs.pop("delete_kwargs", {}))
+        self.insert(document, **update_kwargs.pop("insert_kwargs", {}))
+
     def _preprocess_query(self, mode: QueryMode, query_kwargs: Dict) -> None:
         """Preprocess query.
 
@@ -224,7 +276,7 @@ class BaseGPTIndex(Generic[IS]):
         verbose: bool = False,
         mode: str = QueryMode.DEFAULT,
         **query_kwargs: Any,
-    ) -> str:
+    ) -> Response:
         """Answer a query.
 
         When `query` is called, we query the index with the given `mode` and
