@@ -15,6 +15,7 @@ from typing import Any, Dict, List, Optional
 from gpt_index.data_structs.data_structs import Node
 from gpt_index.indices.common.tree.base import GPTTreeIndexBuilder
 from gpt_index.indices.prompt_helper import PromptHelper
+from gpt_index.indices.response.schema import SourceNode
 from gpt_index.indices.utils import get_sorted_node_list, truncate_text
 from gpt_index.langchain_helpers.chain_wrapper import LLMPredictor
 from gpt_index.prompts.prompts import QuestionAnswerPrompt, RefinePrompt, SummaryPrompt
@@ -27,6 +28,7 @@ class ResponseMode(str, Enum):
     DEFAULT = "default"
     COMPACT = "compact"
     TREE_SUMMARIZE = "tree_summarize"
+    NO_TEXT = "no_text"
 
 
 @dataclass
@@ -60,6 +62,10 @@ class ResponseBuilder:
         """Add text chunk."""
         self._texts.extend(text_chunks)
 
+    def reset(self) -> None:
+        """Clear text chunks."""
+        self._texts = []
+
     def refine_response_single(
         self,
         response: str,
@@ -235,3 +241,26 @@ class ResponseBuilder:
             )
         else:
             raise ValueError(f"Invalid mode: {mode}")
+
+
+class ResponseSourceBuilder:
+    """Response source builder class."""
+
+    # TODO: consolidate with ResponseBuilder
+
+    def __init__(self, nodes: Optional[List[Node]] = None) -> None:
+        """Init params."""
+        nodes = nodes or []
+        self._nodes: List[SourceNode] = SourceNode.from_nodes(nodes)
+
+    def add_node(self, node: Node) -> None:
+        """Add node."""
+        self._nodes.append(SourceNode.from_node(node))
+
+    def add_source_node(self, source_node: SourceNode) -> None:
+        """Add source node directly."""
+        self._nodes.append(source_node)
+
+    def get_sources(self) -> List[SourceNode]:
+        """Get sources."""
+        return self._nodes
